import os, sys
import ctypes
import numpy

from lib.archipelago import Archipelago, Diagnostic_Level

# load bind address/port overrides if specified by arguments
# these are optional parameters; not providing them will result in archipelago library loading them from archipelago.ini config file
bind_addr_override = ""
bind_port_override = 0
if len(sys.argv) > 1:
    bind_addr_override = sys.argv[1]
if len(sys.argv) > 2:
    bind_port_override = sys.argv[2]

# create Archipelago library and load its instance
comm = Archipelago()
result = comm.init_library(Diagnostic_Level.Verbose)
if result == True:
    print("libarchipelago initialized OK")
else:
    print("libarchipelago failed to initialize, exiting")
    exit(1)

# start the communicator - this will establish connections and open socket for incoming connections
result = comm.start_comm(bind_addr_override, bind_port_override)
if result == True:
    print("Communication started successfully")
else:
    print("Failed to start communication, exiting")
    exit(1)


# the following code is there just to demonstrate how to call the archipelago library API
generation = 1
while True:
    try:
        cmd = input("CMD> ").strip()
        if cmd == "exit":
            print("Exiting loop");
            break
        elif cmd == "emit":
            print("Emitting random parameters")

            # DEBUG: generate a random vector and send it
            data = numpy.asarray(numpy.random.rand(12), dtype=numpy.double)
            # DEBUG:
            #print(data)
            comm.broadcast_candidate(data, generation, comm.get_local_guid())
            # DEBUG: increase generation by one to debug the protocol; you may broadcast multiple candidates of a single generation in your code
            generation += 1

            print("Done!")
        elif cmd == "dump":
            print("Receiving foreign candidates...")

            while (comm.has_foreign_candidate()):
                success, candidate, generation, source = comm.pop_foreign_candidate()
                if not success:
                    break

                # here, we have:
                # candidate - numpy.ndarray
                # generation - a number representing the generation of foregitn candidate
                # source - a GUID of candidate origin

                # DEBUG:
                #print(candidate)

            print("Done!")
        elif cmd == "finalize":
            print("Finalizing and synchronizing with all the other islands...")

            # DEBUG: generate random best to send to others
            best_data = numpy.asarray(numpy.random.rand(12), dtype=numpy.double)
            print("Local best")
            print(best_data)
            # you may broadcast multiple best solutions
            comm.broadcast_best(best_data, generation, comm.get_local_guid())

            # this call blocks until all islands call finalize
            comm.finalize()

            while (comm.has_foreign_best()):
                success, best, generation, source = comm.pop_foreign_best()
                if not success:
                    break

                # here, we have:
                # best - numpy.ndarray
                # generation - a number representing the generation of foreign best vector
                # source - a GUID of best vector origin

                # DEBUG:
                #print(best)

            print("Done! Exiting loop")
            break
        else:
            print("Invalid command!")
    except ValueError:
        print("Invalid command!")





# you should always deinitialize the library at the end to allow for correct resource deallocation (global resources and instances, threads, ...)
result = comm.deinit_library()
if result == True:
    print("libarchipelago deinitialized OK")
else:
    print("libarchipelago failed to deinitialize, exiting")
    exit(1)

input("Press enter to continue...")
