# Minimalistic version of comtypes.GUID structure
# The original was dependent on ole32 library, which is present on Windows only
# now we implement the support functions in archipelago library, which is passed as an argument to
# methods that needs some of the support functions; this is done to avoid holding library in memory
# past the point it gets finalized

# Python 3.x+ only

import ctypes

BYTE = ctypes.c_byte
WORD = ctypes.c_ushort
DWORD = ctypes.c_ulong

class GUID(ctypes.Structure):
    _fields_ = [("Data1", DWORD),
                ("Data2", WORD),
                ("Data3", WORD),
                ("Data4", BYTE * 8)]

    def __init__(self, strval=None):
        self.strval = strval
        pass

    def __repr__(self):
        return 'GUID("%s")' % str(self)

    def __unicode__(self):
        return self.strval
    __str__ = __unicode__

    def __cmp__(self, other):
        if isinstance(other, GUID):
            return cmp(bytes(self), bytes(other))
        return -1

    def __bool__(self):
        return self != GUID_null

    def __eq__(self, other):
        return isinstance(other, GUID) and \
               bytes(self) == bytes(other)

    def __hash__(self):
        return hash(bytes(self))

    # updates the stringified version of GUID to internal structure
    # without the call, it holds the old value
    def update_cache(self, utility_lib):
        _StringFromCLSID = utility_lib.support_StringFromCLSID
        _CoTaskMemFree = utility_lib.support_CoTaskMemFree

        p = ctypes.c_wchar_p()
        _StringFromCLSID(ctypes.byref(self), ctypes.byref(p))
        self.strval = p.value
        _CoTaskMemFree(p)


GUID_null = GUID()

__all__ = ["GUID"]
